@@ -70,7 +70,7 @@ pub fn mat_mul<F: PrimeField>(a: &Matrix<F>, b: &Matrix<F>) -> Option<Matrix<F>>
         .iter()
         .map(|input_row| {
             b_t.iter()
-                .map(|transposed_column| vec_mul(&input_row, &transposed_column))
+                .map(|transposed_column| vec_mul(input_row, transposed_column))
                 .collect()
         })
         .collect();
@@ -251,7 +251,7 @@ fn eliminate<F: PrimeField>(
             let mut factor = val;
             factor.mul_assign(&inv_pivot);
 
-            let scaled_pivot = scalar_vec_mul(factor, &pivot);
+            let scaled_pivot = scalar_vec_mul(factor, pivot);
             let eliminated = vec_sub(row, &scaled_pivot);
             result.push(eliminated);
 
@@ -261,13 +261,17 @@ fn eliminate<F: PrimeField>(
             shadow[i] = vec_sub(shadow_row, &scaled_shadow_pivot);
         }
     }
+
+    let pivot_row = shadow.remove(pivot_index);
+    shadow.insert(0, pivot_row);
+
     Some(result)
 }
 
 // `matrix` must be square.
 fn upper_triangular<F: PrimeField>(
     matrix: &Matrix<F>,
-    mut shadow: &mut Matrix<F>,
+    shadow: &mut Matrix<F>,
 ) -> Option<Matrix<F>> {
     assert!(is_square(matrix));
     let mut result = Vec::with_capacity(matrix.len());
@@ -278,7 +282,7 @@ fn upper_triangular<F: PrimeField>(
     while curr.len() > 1 {
         let initial_rows = curr.len();
 
-        curr = eliminate(&curr, column, &mut shadow)?;
+        curr = eliminate(&curr, column, shadow)?;
         result.push(curr[0].clone());
         shadow_result.push(shadow[0].clone());
         column += 1;
@@ -319,8 +323,8 @@ fn reduce_to_identity<F: PrimeField>(
             inv.unwrap()
         };
 
-        let mut normalized = scalar_vec_mul(inv, &row);
-        let mut shadow_normalized = scalar_vec_mul(inv, &shadow_row);
+        let mut normalized = scalar_vec_mul(inv, row);
+        let mut shadow_normalized = scalar_vec_mul(inv, shadow_row);
 
         for j in 0..i {
             let idx = size - j - 1;
@@ -346,7 +350,7 @@ fn reduce_to_identity<F: PrimeField>(
 //
 pub(crate) fn invert<F: PrimeField>(matrix: &Matrix<F>) -> Option<Matrix<F>> {
     let mut shadow = make_identity(columns(matrix));
-    let ut = upper_triangular(&matrix, &mut shadow);
+    let ut = upper_triangular(matrix, &mut shadow);
 
     ut.and_then(|x| reduce_to_identity(&x, &mut shadow))
         .and(Some(shadow))
@@ -459,6 +463,7 @@ mod tests {
 
     #[test]
     fn test_inverse() {
+        let zero = Fr::from(0);
         let one = Fr::from(1);
         let two = Fr::from(2);
         let three = Fr::from(3);
@@ -516,6 +521,13 @@ mod tests {
 
         // S + M(B) = M(B + M^-1(S))
         assert_eq!(add_after_apply, apply_after_add, "breakin' the law");
+
+        let m = vec![vec![zero, one], vec![one, zero]];
+        let m_inv = invert(&m).unwrap();
+        let computed_identity = mat_mul(&m, &m_inv).unwrap();
+        assert!(is_identity(&computed_identity));
+        let computed_identity = mat_mul(&m_inv, &m).unwrap();
+        assert!(is_identity(&computed_identity));
     }
 
     #[test]
